@@ -1,13 +1,9 @@
-import py
+import py, pytest, os
 from pytest import mark, raises, skip
-from .support import setup_make, pylong, IS_WINDOWS, ispypy
+from support import setup_make, pylong, IS_WINDOWS, ispypy
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("advancedcppDict"))
-
-def setup_module(mod):
-    setup_make("advancedcpp")
-    setup_make("advancedcpp2")
+currpath = os.getcwd()
+test_dct = currpath + "/libadvancedcppDict"
 
 
 class TestADVANCEDCPP:
@@ -160,7 +156,7 @@ class TestADVANCEDCPP:
         import cppyy
         gbl = cppyy.gbl
 
-        lib2 = cppyy.load_reflection_info("advancedcpp2Dict")
+        lib2 = cppyy.load_reflection_info("libadvancedcpp2Dict")
 
         assert gbl.a_ns      is gbl.a_ns
         assert gbl.a_ns.d_ns is gbl.a_ns.d_ns
@@ -963,3 +959,7 @@ class TestADVANCEDCPP:
 
         for norm in [ns.norm_cr, ns.norm_m, ns.norm_v]:
             assert round(norm(p3) - pynorm, 8) == 0
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
