@@ -1,12 +1,10 @@
-import py, os, sys
-from pytest import raises
-from .support import setup_make
+import py, sys, pytest, os
+from pytest import mark, raises, skip
+from support import setup_make, ispypy, IS_WINDOWS, IS_MAC_ARM
 
-currpath = py.path.local(__file__).dirpath()
-test_dct = str(currpath.join("doc_helperDict"))
 
-def setup_module(mod):
-    setup_make("doc_helper")
+currpath = os.getcwd()
+test_dct = currpath + "/libdoc_helperDict"
 
 
 class TestDOCFEATURES:
@@ -57,13 +55,13 @@ public:
     void array_method(int* ad, int size) {
         for (int i=0; i < size; ++i)
             std::cerr << ad[i] << ' ';
-        std::cerr << '\\n'; // TODO: not std::endl for 32b Windows
+        std::cerr << std::endl;
     }
 
     void array_method(double* ad, int size) {
         for (int i=0; i < size; ++i)
             std::cerr << ad[i] << ' ';
-        std::cerr << '\\n'; // TODO: not std::endl for 32b Windows
+        std::cerr << std::endl;
     }
 
     void uint_ref_assign(unsigned int& target, unsigned int value) {
@@ -94,6 +92,27 @@ std::string call_abstract_method(Abstract* a) {
     return a->abstract_method();
 }
 
+//-----
+class Abstract1 {
+public:
+    virtual ~Abstract1() {}
+    virtual std::string abstract_method1() = 0;
+};
+
+class Abstract2 {
+public:
+    virtual ~Abstract2() {}
+    virtual std::string abstract_method2() = 0;
+};
+
+std::string call_abstract_method1(Abstract1* a) {
+    return a->abstract_method1();
+}
+
+std::string call_abstract_method2(Abstract2* a) {
+    return a->abstract_method2();
+}
+
 //-----
 int global_function(int) {
     return 42;
@@ -103,7 +122,7 @@ double global_function(double) {
     return std::exp(1);
 }
 
-int call_int_int(int (*f)(int, int), int i1, int i2) {
+int call_int_int_function(int (*f)(int, int), int i1, int i2) {
     return f(i1, i2);
 }
 
@@ -163,7 +182,7 @@ namespace Namespace {
 
         assert cppyy.gbl.gUint == 0
         raises(ValueError, setattr, cppyy.gbl, 'gUint', -1)
-        
+
     def test_casting(self):
         import cppyy
         from cppyy.gbl import Abstract, Concrete
@@ -241,7 +260,7 @@ namespace Namespace {
     def test_functions(self):
         import cppyy
 
-        from cppyy.gbl import global_function, call_int_int, Namespace
+        from cppyy.gbl import global_function, call_int_int_function, Namespace
         assert not(global_function == Namespace.global_function)
 
         assert round(global_function(1.)-2.718281828459045, 8) == 0.
@@ -254,8 +273,8 @@ namespace Namespace {
 
         def add(a, b):
             return a+b
-        assert call_int_int(add, 3, 4) == 7
-        assert call_int_int(lambda x, y: x*y, 3, 7) == 21
+        assert call_int_int_function(add, 3, 4) == 7
+        assert call_int_int_function(lambda x, y: x*y, 3, 7) == 21
 
     def test_inheritance(self):
         import cppyy
@@ -293,9 +312,9 @@ namespace Namespace {
 
     def test_operator_overloads(self):
         import cppyy
-        
+
         pass
-        
+
     def test_pointers(self):
         import cppyy
 
@@ -409,9 +428,32 @@ namespace Namespace {
         pc = PyConcrete4()
         assert call_abstract_method(pc) == "Hello, Python World! (4)"
 
+    @mark.skip
+    def test_multi_x_inheritance(self):
+        """Multiple cross-inheritance"""
+
+        import cppyy
+
+        class PyConcrete(cppyy.multi(cppyy.gbl.Abstract1, cppyy.gbl.Abstract2)):
+            def abstract_method1(self):
+                return "first message"
+
+            def abstract_method2(self):
+                return "second message"
+
+        pc = PyConcrete()
+
+        assert cppyy.gbl.call_abstract_method1(pc) == "first message"
+        assert cppyy.gbl.call_abstract_method2(pc) == "second message"
+
+    @mark.xfail(run=False, condition=IS_MAC_ARM, reason = "Crashes on OS X ARM with" \
+    "libc++abi: terminating due to uncaught exception")
     def test_exceptions(self):
         """Exception throwing and catching"""
 
+        if ispypy:
+            skip('throwing exceptions terminates the process')
+
         import cppyy
 
         caught = False
@@ -433,7 +475,7 @@ namespace Namespace {
             try:
                 cppyy.gbl.DocHelper.throw_an_error(0)
             except exc_type as e:
-                 caught = True
+                caught = True
             assert caught == True
         assert caught == True
 
@@ -764,11 +806,11 @@ class TestADVERTISED:
 
      # enum through void pointer (b/c underlying type unknown)
         vp = ctypes.c_void_p(0); cnt = ctypes.c_int(0)
-        cppyy.gbl.Advert03.build_enum_array2(vp, cnt)
+        cppyy.gbl.Advert03.build_enum_array2(vp, ctypes.pointer(cnt))
         assert cnt.value == 4
 
         vp = ctypes.c_void_p(0); cnt = ctypes.c_int(0)
-        cppyy.gbl.Advert03.build_enum_array1(vp, cnt)
+        cppyy.gbl.Advert03.build_enum_array1(vp, ctypes.pointer(cnt))
         assert cnt.value == 4
 
      # helper to convert the enum array pointer & size to something packaged
@@ -782,8 +824,8 @@ class TestADVERTISED:
         assert list(cppyy.gbl.Advert03.ptr2vec(vp.value, cnt.value)) == [-1, 42, -1, 42]
 
       # 2nd approach through low level cast
-        vp = ctypes.pointer(ctypes.c_uint(0)); cnt = ctypes.c_int(0)
-        cppyy.gbl.Advert03.build_enum_array2(vp, cnt)
+        vp = ctypes.pointer(cppyy.gbl.Advert03.SomeEnum2.__ctype__(0)); cnt = ctypes.c_int(0)
+        cppyy.gbl.Advert03.build_enum_array2(vp, ctypes.pointer(cnt))
         assert cnt.value == 4
 
         import cppyy.ll
@@ -932,3 +974,315 @@ class TestADVERTISED:
         assert n.p[1] == 0x2
         assert n.p[2] == 0x3
         assert len(n.p) == 3
+
+    def test09_custom_str(self):
+        """Example of customized str"""
+
+        import cppyy
+
+        cppyy.cppdef("""\
+        namespace TopologicCore {
+
+        class Shell {
+        public:
+            virtual std::string GetTypeAsString() const {
+                return "hi there!";
+            }
+        }; }""")
+
+
+        def pythonize_topologic_printing(klass, name):
+            if 'GetTypeAsString' in klass.__dict__:
+                klass.__str__ = lambda self: str(self.GetTypeAsString())
+
+        cppyy.py.add_pythonization(pythonize_topologic_printing, 'TopologicCore')
+
+        s = cppyy.gbl.TopologicCore.Shell()
+
+        assert str(s) == "hi there!"
+
+    def test10_llvm_blog(self):
+        """Test code posted in the LLVM blog posting"""
+
+        import cppyy
+
+        cppyy.cppdef(r"""\
+        namespace LLVMBlog {
+        template<typename T> class Producer {
+        public:
+            Producer(const T& value) : m_value(value) {}
+            virtual ~Producer() {}
+
+            T produce_total() { return m_value + produce_imp(); }
+
+        protected:
+            virtual T produce_imp() = 0;
+
+        private:
+            T m_value;
+        };
+
+        class Consumer {
+        public:
+            template<typename T>
+            std::string consume(Producer<T>& p) {
+                std::ostringstream s;
+                s << "received: \"" << p.produce_total() << "\"";
+                return s.str();
+            }
+        }; } """)
+
+        ns = cppyy.gbl.LLVMBlog
+
+        def factory(base_v, *derived_v):
+            class _F(ns.Producer[type(base_v)]):
+                def __init__(self, base_v, *derived_v):
+                    if sys.hexversion < 0x3000000:
+                        super(type(self), self).__init__(base_v)
+                    else:
+                        super().__init__(base_v)
+                    self._values = derived_v
+
+                def produce_imp(self):
+                    return type(base_v)(sum(self._values))
+
+            return _F(base_v, *derived_v)
+
+        consumer = ns.Consumer()
+
+        assert consumer.consume(factory("hello ", 42))     == 'received: "hello 42"'
+        assert consumer.consume(factory(3., 0.14, 0.0015)) == 'received: "3.1415"'
+
+
+# The series of tests below mostly exists already in other places, but these
+# were used as examples for the CaaS' cppyy presentation and are preserved here.
+class TestTALKEXAMPLES:
+    def setup_class(cls):
+        import cppyy
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        struct MyClass {
+            MyClass(int i) : fData(i) {}
+            virtual ~MyClass() {}
+            virtual int add(int i) {
+                return fData + i;
+            }
+            int fData;
+        };}""")
+
+        cppyy.gbl.talk_examples
+
+    def test_template_instantiation(self):
+        """Run-time template instantiation example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        v = cppyy.gbl.std.vector[CC.MyClass]()
+
+        for i in range(10):
+            v.emplace_back(i)
+
+        assert len(v) == 10
+        assert [m.fData for m in v] == list(range(10))
+
+    def test_cross_inheritance(self):
+        """Cross-inheritance example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        int callb(MyClass* m, int i) {
+          return m->add(i);
+        }}""")
+
+        class PyMyClass(CC.MyClass):
+            def add(self, i):
+                return self.fData + 2*i
+
+        m = PyMyClass(1)
+        assert CC.callb(m, 2) == 5
+
+    def test_cross_and_templates(self):
+        """Template instantiation with cross-inheritance example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        class PyMyClass(CC.MyClass):
+            def __init__(self, data, extra):
+                super(PyMyClass, self).__init__(data)
+                self.extra = extra
+
+            def add(self, i):
+                return self.fData + self.extra + 2*i
+
+        v = cppyy.gbl.std.vector[PyMyClass]()
+        v.push_back(PyMyClass(4, 42))
+
+        assert v.back().add(17) == 4+42+2*17
+
+    @mark.xfail()
+    def test_fallbacks(self):
+        """Template instantation switches based on value sizes"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        template<typename T>
+        T passT(T t) {
+            return t;
+        }}""")
+
+        assert CC.passT(1) == 1
+        assert 'int' in CC.passT.__doc__
+        assert CC.passT(2**64-1) == 2**64-1
+        assert 'unsigned long long' in CC.passT.__doc__
+
+    def test_callbacks(self):
+        """Function callback example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        typedef int (*P)(int);
+        int callPtr(P f, int i) {
+            return f(i);
+        }
+
+        typedef std::function<int(int)> F;
+        int callFun(const F& f, int i) {
+            return f(i);
+        }}""")
+
+        def f(val):
+            return 2*val
+
+        assert CC.callPtr(f, 2) == 4
+        assert CC.callFun(f, 3) == 6
+        assert CC.callPtr(lambda i: 5*i, 4) == 20
+        assert CC.callFun(lambda i: 6*i, 4) == 24
+
+    @mark.xfail()
+    def test_templated_callback(self):
+        """Templated callback example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        template<typename R, typename... U, typename... A>
+        R callT(R (*f)(U...), A&&... args) {
+            return f(args...);
+        }}""")
+
+        if sys.hexversion < 0x3050000:
+            def ann_f1(arg):
+                return 3.1415*arg
+            ann_f1.__annotations__ = {'arg': 'int', 'return': 'double'}
+            def ann_f2(arg1, arg2):
+                return 3*arg1*arg2
+            ann_f2.__annotations__ = {'arg1': 'int', 'arg2' : 'int', 'return': 'int'}
+        else:
+            oldp = sys.path[:]
+            sys.path.append('.')
+            from doc_args_funcs import ann_f1, ann_f2
+            sys.path = oldp
+
+        assert round(CC.callT(ann_f1, 2)-2*3.1415, 5) == 0.
+
+        assert CC.callT(ann_f2, 6, 7) == 3*6*7
+        assert round(CC.callT(ann_f1, 2)-2*3.1415, 5) == 0.
+
+    def test_autocast_and_identiy(self):
+        """Auto-cast and identiy preservation example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        struct Base {
+            virtual ~Base() {}
+        };
+        struct Derived : public Base {};
+        Base* passB(Base* b) { return b; }
+        }""")
+
+        d = CC.Derived()
+        b = CC.passB(d)
+
+        assert type(b) == CC.Derived
+        assert d is b
+
+    @mark.xfail(run=False, condition=IS_MAC_ARM, reason = "Crashes on OS X ARM with" \
+    "libc++abi: terminating due to uncaught exception")
+    def test_exceptions(self):
+        """Exceptions example"""
+
+        if ispypy or IS_WINDOWS:
+            skip('throwing exceptions from the JIT terminates the process')
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        class MyException : public std::exception {
+        public:
+           const char* what() const throw() {
+               return "C++ failed";
+           }
+        };
+        void throw_error() {
+            throw MyException{};
+        }}""")
+
+        with raises(CC.MyException):
+            CC.throw_error()
+
+    @mark.xfail()
+    def test_unicode(self):
+        """Unicode non-UTF-8 example"""
+
+        import cppyy
+        import cppyy.gbl.talk_examples as CC
+
+        cppyy.cppdef("""\
+        namespace talk_examples {
+        template<class T>
+        std::string to_str(const T& chars) {
+            char buf[12]; int n = 0;
+            for (auto c : chars) buf[n++] = char(c);
+            return std::string(buf, n-1);
+        }
+        std::string utf8_chinese() {
+            auto chars = {0xe4, 0xb8, 0xad, 0xe6, 0x96, 0x87, 0};
+            return to_str(chars);
+        }
+        std::string gbk_chinese() {
+            auto chars = {0xd6, 0xd0, 0xce, 0xc4, 0};
+            return to_str(chars);
+        }}""")
+
+        with raises(Exception) as exc_info:
+            CC.gbk_chinese().decode('utf-8')
+        assert isinstance(exc_info.value, (UnicodeDecodeError, LookupError))
+
+        assert CC.gbk_chinese() == u'\u4e2d\u6587'.encode('gbk')
+        if 0x3000000 <= sys.hexversion:
+            assert CC.utf8_chinese() == u'\u4e2d\u6587'
+        else:
+            assert CC.utf8_chinese() == b'\xe4\xb8\xad\xe6\x96\x87'
+
+
+if __name__ == "__main__":
+    exit(pytest.main(args=['-sv', '-ra', __file__]))
