@@ -100,9 +100,6 @@ TGButtonGroup.
 #include <iostream>
 
 
-ClassImp(TGButtonGroup);
-ClassImp(TGHButtonGroup);
-ClassImp(TGVButtonGroup);
 
 ////////////////////////////////////////////////////////////////////////////////
 /// Constructor. Layout 1 row or 1 column.
