@@ -1,14 +1,17 @@
 #ifndef CPPINTEROP_UNITTESTS_LIBCPPINTEROP_UTILS_H
 #define CPPINTEROP_UNITTESTS_LIBCPPINTEROP_UTILS_H
 
-#include "../../lib/Interpreter/Compatibility.h"
+#include "../../lib/CppInterOp/Compatibility.h"
 
-#include "llvm/Support/Valgrind.h"
-#include <memory>
-#include <vector>
 #include "clang-c/CXCppInterOp.h"
 #include "clang-c/CXString.h"
 
+#include "llvm/Support/Valgrind.h"
+
+#include <memory>
+#include <string>
+#include <vector>
+
 using namespace clang;
 using namespace llvm;
 
@@ -17,10 +20,12 @@ namespace clang {
 }
 #define Interp (static_cast<compat::Interpreter*>(Cpp::GetInterpreter()))
 namespace TestUtils {
-  void GetAllTopLevelDecls(const std::string& code, std::vector<clang::Decl*>& Decls,
-                           bool filter_implicitGenerated = false);
-  void GetAllSubDecls(clang::Decl *D, std::vector<clang::Decl*>& SubDecls,
-                      bool filter_implicitGenerated = false);
+void GetAllTopLevelDecls(const std::string& code,
+                         std::vector<clang::Decl*>& Decls,
+                         bool filter_implicitGenerated = false,
+                         const std::vector<const char*>& interpreter_args = {});
+void GetAllSubDecls(clang::Decl* D, std::vector<clang::Decl*>& SubDecls,
+                    bool filter_implicitGenerated = false);
 } // end namespace TestUtils
 
 const char* get_c_string(CXString string);
