@@ -53,6 +53,7 @@
 #include "llvm/IR/LLVMContext.h"
 #include "llvm/IR/Module.h"
 #include "llvm/Support/CrashRecoveryContext.h"
+#include "llvm/Support/Error.h"
 #include "llvm/Support/MemoryBuffer.h"
 
 #include <stdio.h>
@@ -204,7 +205,7 @@ namespace {
   public:
     FilteringDiagConsumer(DiagnosticsEngine& Diags, bool Own)
       : DiagnosticsOverride(Diags, Own) {}
-    virtual ~FilteringDiagConsumer() { setTargetConsumer(nullptr); }
+    ~FilteringDiagConsumer() override { setTargetConsumer(nullptr); }
 
     /// \brief Sets the DiagnosticConsumer that sees `HandleDiagnostic()` calls.
     /// \param[in] Consumer - The target DiagnosticConsumer, or `nullptr` to
@@ -911,8 +912,11 @@ namespace cling {
     PP.EnterSourceFile(FID, /*DirLookup*/nullptr, NewLoc);
     m_Consumer->getTransaction()->setBufferFID(FID);
 
-    if (!ParseOrWrapTopLevelDecl())
+    llvm::Error res = ParseOrWrapTopLevelDecl();
+    if (res) {
+      llvm::consumeError(std::move(res));
       return kFailed;
+    }
 
     if (PP.getLangOpts().DelayedTemplateParsing) {
       // Microsoft-specific:
@@ -939,7 +943,7 @@ namespace cling {
     return kSuccess;
   }
 
-  llvm::Expected<bool> IncrementalParser::ParseOrWrapTopLevelDecl() {
+  llvm::Error IncrementalParser::ParseOrWrapTopLevelDecl() {
     // Recover resources if we crash before exiting this method.
     Sema& S = getCI()->getSema();
     DiagnosticsEngine& Diags = getCI()->getDiagnostics();
@@ -993,7 +997,7 @@ namespace cling {
       // Let's ignore this transaction:
       m_Consumer->getTransaction()->setIssuedDiags(Transaction::kErrors);
 
-      return true;
+      return llvm::Error::success();
     }
 
     // Process any TopLevelDecls generated by #pragma weak.
@@ -1005,7 +1009,7 @@ namespace cling {
     LocalInstantiations.perform();
     GlobalInstantiations.perform();
 
-    return true;
+    return llvm::Error::success();
   }
 
   void IncrementalParser::printTransactionStructure() const {
@@ -1029,9 +1033,11 @@ namespace cling {
       // Don't protect against crashes if we cannot run anything.
       // cling might also be in a PCH-generation mode; don't inject our Sema
       // pointer into the PCH.
+#ifndef CLING_WITH_ADAPTIVECPP
       if (!isCUDADevice && m_Interpreter->getOptions().PtrCheck)
         ASTTransformers.emplace_back(
             new NullDerefProtectionTransformer(m_Interpreter));
+#endif
       if (isCUDADevice)
         ASTTransformers.emplace_back(
             new DeviceKernelInliner(TheSema));
