@@ -50,7 +50,7 @@ public:
    virtual ~FitTransformFunction() {}
 
    // re-implement data element
-   virtual double DataElement(const double *x, unsigned i, double *g = nullptr, double * = nullptr, bool = false) const
+   double DataElement(const double *x, unsigned i, double *g = nullptr, double * = nullptr, bool = false) const override
    {
       // transform from x internal to x external
       const double *xExt = fTransform->Transformation(x);
@@ -63,18 +63,18 @@ public:
       return val;
    }
 
-   virtual IMultiGenFunction *Clone() const
+   IMultiGenFunction *Clone() const override
    {
       // not supported
       return nullptr;
    }
 
    // dimension (this is number of free dimensions)
-   virtual unsigned int NDim() const { return fTransform->NDim(); }
+   unsigned int NDim() const override { return fTransform->NDim(); }
 
    unsigned int NTot() const { return fTransform->NTot(); }
 
-   virtual typename FMFunc::Type_t Type() const { return fFunc.Type(); }
+   typename FMFunc::Type_t Type() const override { return fFunc.Type(); }
 
    // forward of transformation functions
    const double *Transformation(const double *x) const { return fTransform->Transformation(x); }
@@ -104,9 +104,9 @@ private:
    FitTransformFunction(const FitTransformFunction &rhs) = delete;
    FitTransformFunction &operator=(const FitTransformFunction &rhs) = delete;
 
-   virtual double DoEval(const double *x) const { return fFunc(fTransform->Transformation(x)); }
+   double DoEval(const double *x) const override { return fFunc(fTransform->Transformation(x)); }
 
-   virtual double DoDerivative(const double * /* x */, unsigned int /*icoord*/) const
+   double DoDerivative(const double * /* x */, unsigned int /*icoord*/) const override
    {
       // not used
       throw std::runtime_error("FitTransformFunction::DoDerivative");
