@@ -19,17 +19,17 @@
 
 #include <ROOT/RConfig.hxx>
 
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstring>
+#include <cstdlib>
 #include <unistd.h>
-#include <signal.h>
+#include <csignal>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-#include <errno.h>
+#include <cerrno>
 
 #include "rpdp.h"
 #include "rpderr.h"
@@ -472,8 +472,6 @@ int NetInit(EService servtype, int port1, int port2, int tcpwindowsize)
          if (!sp) {
             if (servtype == kROOTD) {
                port1 = 1094;
-            } else if (servtype == kPROOFD) {
-               port1 = 1093;
             } else {
                fprintf(stderr,"NetInit: unknown service: %s/tcp\n", service.data());
                Error(gErrFatal, kErrFatal,
