@@ -46,9 +46,6 @@
 #include "RooProfileLL.h"
 #include "TF1.h"
 
-/// ClassImp for building the THtml documentation of the class
-ClassImp(RooStats::LikelihoodIntervalPlot);
-
 using namespace RooStats;
 
 ////////////////////////////////////////////////////////////////////////////////
